/// <reference types="node" />
import { type ProcessOutput } from 'zx';
export { type Client as WatchmanClient } from 'fb-watchman';
type RelationalOperator = 'eq' | 'ge' | 'gt' | 'le' | 'lt' | 'ne';
type FileType = '?' | 'b' | 'c' | 'D' | 'd' | 'f' | 'l' | 'p' | 's';
type Expression = ['allof', ...Expression[]] | ['anyof', ...Expression[]] | ['dirname' | 'idirname', string, ['depth', RelationalOperator, number]] | ['empty'] | ['exists'] | ['match' | 'imatch', string | string[], 'basename' | 'wholename'] | ['name', string, 'basename' | 'wholename'] | ['not', Expression] | ['pcre' | 'ipcre', string, 'basename' | 'wholename'] | ['since', string | number, 'mtime' | 'ctime', 'oclock'] | ['size', RelationalOperator, number] | ['suffix', string | string[]] | ['type', FileType];
type JsonValue = JsonObject | JsonValue[] | boolean | number | string | readonly JsonValue[] | null | undefined;
export type JsonObject = {
    [k: string]: JsonValue;
};
/**
 * @property mtime The timestamp indicating the last time this file was modified.
 */
type File = {
    exists: boolean;
    mtime: number;
    name: string;
    size: number;
};
/**
 * @property attempt Attempt number (starting with 0) indicating if trigger was retried.
 * @property files Describes the list of files that changed.
 * @property first Identifies if this is the first event.
 * @property signal Instance of AbortSignal used to signal when the routine should be aborted.
 * @property spawn Instance of zx bound to AbortSignal.
 * @property warning Watchman warnings.
 */
export type ChangeEvent = {
    abortSignal?: AbortSignal;
    attempt: number;
    files: readonly File[];
    first: boolean;
    spawn: (pieces: TemplateStringsArray, ...args: any[]) => Promise<ProcessOutput>;
    taskId: string;
    warning: string | null;
};
type OnChangeEventHandler = (event: ChangeEvent) => Promise<any>;
export type TeardownEvent = {
    spawn: (pieces: TemplateStringsArray, ...args: any[]) => Promise<ProcessOutput>;
};
type OnTeardownEventHandler = (event: TeardownEvent) => Promise<void>;
/**
 * @property factor The exponential factor to use. Default is 2.
 * @property maxTimeout The maximum number of milliseconds between two retries. Default is Infinity.
 * @property minTimeout The number of milliseconds before starting the first retry. Default is 1000.
 * @property retries The maximum amount of times to retry the operation. Default is 10. Seting this to 1 means do it once, then retry it once.
 */
type Retry = {
    factor?: number;
    maxTimeout?: number;
    minTimeout?: number;
    retries?: number;
};
type Debounce = {
    leading?: boolean;
    wait: number;
};
export type Throttle = {
    delay: number;
};
/**
 * @property expression watchman expression, e.g. https://facebook.github.io/watchman/docs/expr/allof.html
 * @property interruptible Sends abort signal to an ongoing routine, if any. Otherwise, waits for routine to finish. (default: true)
 * @property name Name of the trigger. Used for debugging. Must match /^[a-z0-9-_]+$/ pattern and must be unique.
 * @property onChange Routine that is executed when file changes are detected.
 * @property persistent Label a task as persistent if it is a long-running process, such as a dev server or --watch mode.
 */
type TriggerInput = {
    debounce?: Debounce;
    expression: Expression;
    interruptible?: boolean;
    name: string;
    onChange: OnChangeEventHandler;
    onTeardown?: OnTeardownEventHandler;
    persistent?: boolean;
    retry?: Retry;
    throttleOutput?: Throttle;
};
export type Trigger = {
    abortSignal?: AbortSignal;
    debounce?: Debounce;
    expression: Expression;
    id: string;
    interruptible: boolean;
    name: string;
    onChange: OnChangeEventHandler;
    onTeardown?: OnTeardownEventHandler;
    relativePath: string;
    retry: Retry;
    throttleOutput: Throttle;
    watch: string;
};
/**
 * @property project absolute path to the directory to watch
 */
export type ConfigurationInput = {
    readonly abortSignal?: AbortSignal;
    readonly project: string;
    readonly triggers: readonly TriggerInput[];
};
export type Configuration = {
    readonly abortSignal?: AbortSignal;
    readonly project: string;
    readonly triggers: readonly TriggerInput[];
};
